# Import SubCell
from subcell import *


# Default parameters all accessible from the command prompt
default_params = ParameterDict(
    tstop = RangeCheck(30, gt=0),
    dt_max = RangeCheck(0.5, gt=0, le=5.),
    dt_start = RangeCheck(0.0001, gt=0, le=5.),
    dt_scale = RangeCheck(1.2, gt=1, le=5.),
    num_runs = RangeCheck(1, ge=1),
    simple = True, 
    )

def main(params):
    """The main function which load and runs the model"""

    # Create the timestepper
    timestepper = TimeStepper("channel-NCX", simple=params.simple)
    p = timestepper.params
    
    # Set casename
    p.io.casename = "channel-NCX"
    p.io.use_logfile = True
    
    # Set parameters
    p.diffreact.io.field_species.outerCa_innerCa.plot = True
    p.diffreact.io.field_species.outerCa_innerCa.vmax = 10
    p.diffreact.io.field_species.outerNa_innerNa.plot = True
    p.diffreact.io.field_species.outerNa_innerNa.vmax = 30000

    p.diffreact.io.conservation = ["Ca", "Na"]
    
    # Solver parameters
    p.diffreact.newton_solver.absolute_tolerance = 1e-6
    p.diffreact.newton_solver.relative_tolerance = 1e-6
    
    # Set dolfin thread parameter
    p.diffreact.dolfin.num_threads = 4
    
    # Time stepping parameters
    p.time.tstop = params.tstop
    p.time.dt_max = params.dt_max
    p.time.dt_start = params.dt_start
    p.time.num_runs = params.num_runs

    # Print Parameters
    print p
    
    # Just do it
    timestepper.run()
    
if __name__ == "__main__":
    default_params.parse_args()
    main(default_params)

