"""
A small test example for channel flow with pump
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-16 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(
    sigma=RangeCheck(2.2e5, ge=0.1, le=1.0e7),
    Vmax = RangeCheck(286., gt=0.),
    Kmf = RangeCheck(0.246, gt=0.),
    Kmr = RangeCheck(1700., gt=0.),
    H = RangeCheck(1.787, gt=0.),
    Q10 = RangeCheck(2.6, gt=0.),
    VolumeToArea = RangeCheck(400., gt=0.),
    )

mesh = Mesh("channel-mesh")
Cytosol = MeshDomain("Cytosol", mesh, 2)
SR = MeshDomain("SR", mesh, 3)

channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.])

# Arbitrary parameters can be declared
channel = MeshBoundary("channel", (Cytosol, SR), (10, 20), 1.0)

Ca = Species("Ca", 2)

ScalarFieldSpecies(Cytosol, Ca, 5., sigma)
ScalarFieldSpecies(SR, Ca, 1300, sigma)

# Add SERCA pump flux from
# Shannon, T. R.; Wang, F.; Puglisi, J.; Weber, C. & Bers, D. M.
# A mathematical treatment of integrated Ca dynamics within the
# ventricular myocyte.
# Biophys. J., 2004, 87, 3351-3371
Flux("SERCA", channel, Ca, Q10*Vmax*((Cytosol.Ca/Kmf)**H-(SR.Ca/Kmr)**H)/(1+(Cytosol.Ca/Kmf)**H+(SR.Ca/Kmr)**H))

# Add a small reverse version of the flux
DiffusionFlux(channel, Ca, sigma/10000)
