"""
A small test example for channel flow and disipation out of the container
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-12-01 -- 2011-09-15"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameters
ModelParameters(c0=RangeCheck(1., ge=0.1, le=1.0e7),
                sigma=RangeCheck(2., ge=0.1, le=1.0e7),
                amp=50., dx=1.0)

# Declare domains
mesh = Mesh("channel-mesh-0")
lower = MeshDomain("lower", mesh, 2)
cytosol = InfiniteDomain("cytosol")

# Declare boundaries
channel = MeshBoundary("channel", lower, 10, 1)
to_cytosol = MeshBoundary("to_cytosol", (lower, cytosol), 2, dx)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.], True)

# Connect controller with Boundary
Connection(channel_ctrl, channel)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma*5)
ConstSpecies(cytosol, Ca, c0)

# Declare fluxes on Boundaries
Flux("CaChannel", channel, Ca, amp)
DiffusionFlux(to_cytosol, Ca, sigma)
