"""
A small test example for channel flow with fixed values
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2012-04-27 -- 2012-04-27"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Domains
mesh = Mesh("channel-mesh-0")
lower = MeshDomain("lower", mesh, 2)
upper  = InfiniteDomain("upper")

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), 10, 1, sigma=sigma)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.], True)
Connection(channel_ctrl, channel)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma*5)
ConstSpecies(upper, Ca, 10.)

# Declare fixed value on Boundary
FixedValue(channel, lower, Ca, upper.Ca)

