"""
A small test example for channel flow
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-26 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Domains
mesh = Mesh("channel-mesh-0")
lower = MeshDomain("lower", mesh, 2)
upper  = VolumeDomain("upper", 1000.)

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), 10, 1, sigma=sigma)

# Boundary controller
channel_ctrl = OneTimeValueCrosser("channel_ctrl", 7.0, minimize_dt=True)
Connection(channel_ctrl, channel)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma*5)
ScalarSpecies(upper, Ca, 10.)

# Connect controller with Boundary
Connection(upper.Ca, channel_ctrl)

# Declare flux on Boundary
DiffusionFlux(channel, Ca, channel.sigma)

