
// gillstep generated Markov model code

#ifndef __CA_LOWER_BIND_H
#define __CA_LOWER_BIND_H

#include <vector>
#include <map>
#include <cmath>
#include <cassert>
#include <stdexcept>
#include <algorithm>
#include <boost/bind.hpp>

#include <gillstep/common.h>
#include <gillstep/MarkovModel.h>
#include <gillstep/GillespieStepper.h>
#include <gillstep/temperature.h>

namespace gillstep
{

  /// Markov model 
  class Ca_lower_bind : public MarkovModel
  {
      
  
  public:

    /// Default constructor
    Ca_lower_bind(GillespieStepper& gillstepper) :
        MarkovModel("Ca_lower_bind", gillstepper),
        _present_substep(0.0),
        _variables_05(1),
        _rate_05(1),
        _rate_deriviatives_05(1)
        
    {
      /// Resize field
      _variables_0.resize(1);
      _variables_1.resize(1);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["Ca"] = boost::bind(&Ca_lower_bind::_receive_Ca, this, _1);
      _variable_map["Ca"] = 0;
    }
    
    /// SubMarkov model constructor
    Ca_lower_bind(MarkovModel* parent, GillespieStepper& gillstepper) :
        MarkovModel("Ca_lower_bind", gillstepper),
        _present_substep(0.0),
        _variables_05(1),
        _rate_05(1),
        _rate_deriviatives_05(1)
    {
      // Assign parent pointer
      _parent = parent;

      /// Resize field
      _variables_0.resize(1);
      _variables_1.resize(1);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["Ca"] = boost::bind(&Ca_lower_bind::_receive_Ca, this, _1);
      _variable_map["Ca"] = 0;

      // Check if model can receive "parent"
      if (can_receive(parent->name()))
      {
        parent->connect(receive(parent->name()));
        _receive_parent = true;
      }
    
    }
    
    /// Destructor
    virtual ~Ca_lower_bind(){};
    
    /// Returns the maximal transitions from a state
    uint max_transitions() const
    {
       return 1;
    }

    /// Return the 'value' of the Markov model the state represent 
    double value() const
    {
      switch (_state)
      {
        case 1:
          return 1;
      default:
        return 0.0;
      }
    
    }

    /// Return the size of the substep. 1.0 if no substep is needed
    double substep_size(double present_substep)
    {
      assert(present_substep >= 0.0);
        
      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep)
      {
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      double next_substep = 1;
      for (uint trans = 0; trans < _transition_map[_state].size(); trans++)
      {
        if (std::fabs(_rate_deriviatives_05[trans]) > 1e-10)
            next_substep = std::min(next_substep, std::fabs(
                           _gillstepper->get_max_fractional_rate_change()*
                           (_rate_05[trans]/_rate_deriviatives_05[trans])));
      }
      
      // Update the the next substep result with the present one
      next_substep = present_substep + next_substep;

      // Return next substep. Return 1.0 if it is larger than 1.0
      return next_substep > 1.0 ? 1.0 : next_substep;
    }

    /// Evaluate rate function corresponding to passed transition
    double evaluate_rate(uint transition) const
    {
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=Ca_lower_bind::_transition_map[_state].size())
        return 0.0;

      // Evaluate the rate
      return Ca_lower_bind::_rate_functions[_state][transition](*this);
    }
    
    /// Evaluate rate function corresponding to passed state and transition
    double evaluate_rate(uint from_state, uint to_state) const
    {
      switch (from_state)
      {
        case 0:
          switch (to_state) {
            case 1:
              return Ca_lower_bind::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
        case 1:
          switch (to_state) {
            case 0:
              return Ca_lower_bind::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
      default:
        return 0.0;
      }
    }
    
    /// Evaluate rate function corresponding to passed transition for a substep
    double evaluate_substep_rate(uint transition, double present_substep,
                                 double next_substep)
    {
      assert(present_substep>=0);
      assert(next_substep>=present_substep);
      
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=Ca_lower_bind::_transition_map[_state].size())
        return 0.0;

      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      // Use tabulated values
      return _rate_05[transition] + (next_substep-_present_substep)*
        (_rate_deriviatives_05[transition]);
    }
    
    /// Change the state to the given transition
    bool change_state(int transition)
    {
      /// If forced to not change the value of this Markov Model
      if (fix_state)
        return false;
        
      /// We need to retabulate derivatives
      _retabulate_derivatives = true;
        
      const int num_transitions = Ca_lower_bind::_transition_map[_state].size();
      assert(transition<num_transitions);
        
      /// Update the state from the transition
      if (transition>=0)
        _state = Ca_lower_bind::_transition_map[_state][transition];

      /// If this is a SubMarkov model return the parents change_state value
      if (_parent)
        return _parent->change_state(-1);

      /// If this is not a SubMarkov model check if my value has change
      const bool changed = _old_value != value();
      _old_value = value();
      return changed;
    }

    /// Get temperature scale
    static inline double temp_scale()
    {
      return Ca_lower_bind::_temp_scale;
    }

    /// Get parameter
    static inline double get_parameter(uint ind)
    {
      return Ca_lower_bind::_params[ind];
    }

    /// Get parameter
    double get_parameter(const std::string& name) const
    {
      return _params[_find_parameter(name)];
    }
    
    /// Set parameter
    void set_parameter(const std::string& name, double value)
    {
      //std::cout << "Setting " << name << " to " << value << std::endl;
      _params[_find_parameter(name)] = value;
      // If the temperature scale is change update it
      if (name == "T_ref" or name == "Q10")
        update_temp_scale();
    }

    /// Get parameter names
    virtual std::vector<std::string> get_parameter_names() const
    {
      std::vector<std::string> parameter_names;

      parameter_names.push_back("Q10");
      parameter_names.push_back("T_ref");
      parameter_names.push_back("k_off");
      parameter_names.push_back("k_on");

      // Return by value
      return parameter_names;
    };

    /// Return the number of states
    inline uint num_states() const
    {
      return 2;
    }

    /// Update the temperature scale factor
    void update_temp_scale()
    {
      Ca_lower_bind::_temp_scale = pow(Ca_lower_bind::_params[0], ((get_T()-T_abs())-Ca_lower_bind::_params[1])/10);
    }

  private:

    /// Tabulate rate derivatives
    void _tabulate_rate_deriviatives()
    {
      // Local variables
      uint trans, var;

      // Update variables at present substep
      for (var=0; var < _variables_05.size(); var++)
      {
        _variables_05[var] = _variables_1[var]*_present_substep +
                             (1-_present_substep)*_variables_0[var];
        //std::cout << "Variable: " << _variables_05[var] << std::endl;
      }
        
      // Set the present variables to present interpolated values
      _present_variables = &_variables_05;

      for (trans=0; trans < _transition_map[_state].size(); trans++)
      {
        _rate_05[trans] = std::max(0.1, Ca_lower_bind::_rate_functions
                                   [_state][trans](*this));
        //std::cout << "Rate: " << _rate_05[var] << std::endl;
        _rate_deriviatives_05[trans] = 0.0;
        for (var=0; var<_variables_0.size(); var++){
          _rate_deriviatives_05[trans] += Ca_lower_bind::_rate_derivative_functions
                                         [_state][trans][var](*this)*
                                         (_variables_1[var] - _variables_05[var]);
        //std::cout << "Rate der: " << _rate_deriviatives_05[var] << std::endl;
        }
      }

      // Set the present variables to the last received values
      _present_variables = &_variables_1;

      // Set flag preventing unessesary tabulation
      _retabulate_derivatives = false;
    }

    /// Check parameter name
    static uint _find_parameter(const std::string& name)                              
    {
      std::map<std::string, uint>::const_iterator it = Ca_lower_bind::_param_map.find(name);
      if (it == Ca_lower_bind::_param_map.end())
        throw std::runtime_error("\"" + name + "\" is not a parameter in this Markov Model");                         
      return it->second;
    }

    
    /// Receive methods
    inline void _receive_Ca(double value)
    {
      _retabulate_derivatives = true;
      _variables_0[0] = _variables_1[0];
      _variables_1[0] = value;
    }
    

    /// Data members

    // Keep track of which substep we are on
    double _present_substep;

    // Variables values inside time step
    std::vector<double> _variables_05;

    /// The interpolated rate values and its derivatives wrt to the variables
    std::vector<double> _rate_05;
    std::vector<double> _rate_deriviatives_05;

      
    /// Temperature scale factor
    static double _temp_scale;

    /// An array for the parameter values
    static std::vector<double> _params;

    /// Mapping names to parameter indices
    static const std::map<std::string, uint> _param_map;
    
    /// Mapping state and transition number to state
    static const std::vector<std::vector<uint> > _transition_map;

    /// Mapping state and transition number to rate function
    static const std::vector<std::vector<rate_call> > _rate_functions;

    /// Mapping state, transition number and continuous variables to rate
    /// derivative function
    static const std::vector<std::vector<std::vector<rate_call> > > _rate_derivative_functions;
    
  };

  // Name space for hiding static initalization functions
  namespace InitCa_lower_bind
  {

    /// Define Rate functions
    
    /// Rate function between state: 0 and 1 
    double rate_func_0_1(const MarkovModel& mm)
    {
      const double value = Ca_lower_bind::temp_scale()*(Ca_lower_bind::get_parameter(3)*mm._unsafe_variable(0));
    //  std::cout << mm.name() << " tau: 0-1: " << 1/value << std::endl;
      return value;
    }
    
    
    /// Rate function between state: 1 and 0 
    double rate_func_1_0(const MarkovModel& mm)
    {
      const double value = Ca_lower_bind::temp_scale()*(Ca_lower_bind::get_parameter(2));
    //  std::cout << mm.name() << " tau: 1-0: " << 1/value << std::endl;
      return value;
    }
    
    

    /// Define derivatives of Rate functions
    
    /// Derivative of rate function between state: 0 and 1 for variable: Ca
    double der_rate_func_0_1_wrt_Ca(const MarkovModel& mm)
    {
      return Ca_lower_bind::temp_scale()*(Ca_lower_bind::get_parameter(3));
    }
    
    
    /// Derivative of rate function between state: 1 and 0 for variable: Ca
    double der_rate_func_1_0_wrt_Ca(const MarkovModel& mm)
    {
      return Ca_lower_bind::temp_scale()*(0);
    }
    
    
    /// Init params
    std::vector<double> params()
    {
      // vector which will be returned by value
      std::vector<double> params_;
      params_.push_back(1.000000);
      params_.push_back(37.000000);
      params_.push_back(1.000000);
      params_.push_back(0.500000);

      // return by value
      return params_;
    }

    /// Init param_map
    const std::map<std::string, uint> param_map()
    {
      // map which will be returned by value
      std::map<std::string, uint> param_map_;
      param_map_["Q10"] = 0;
      param_map_["T_ref"] = 1;
      param_map_["k_off"] = 2;
      param_map_["k_on"] = 3;

      // return by value
      return param_map_;
    }

    /// Init transition_map
    const std::vector<std::vector<uint> > transition_map()
    {
      // vector which will be returned by value
      std::vector<std::vector<uint> > transition_map_;
      
      // Transition map from state 0
      std::vector<uint> transition_map_0;
      transition_map_0.push_back(1);
      transition_map_.push_back(transition_map_0);
      
      // Transition map from state 1
      std::vector<uint> transition_map_1;
      transition_map_1.push_back(0);
      transition_map_.push_back(transition_map_1);

      // return by value
      return transition_map_;
    }

    /// Init rate_functions
    const std::vector<std::vector<rate_call> > rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<rate_call> > rate_functions_;
      
      // Rate functions from state 0
      std::vector<rate_call> rate_functions_0;
      rate_functions_0.push_back(rate_func_0_1);
      rate_functions_.push_back(rate_functions_0);
      
      // Rate functions from state 1
      std::vector<rate_call> rate_functions_1;
      rate_functions_1.push_back(rate_func_1_0);
      rate_functions_.push_back(rate_functions_1);

      // return by value
      return rate_functions_;
    }
    
    /// Init derivative of rate_functions
    const std::vector<std::vector<std::vector<rate_call> > > der_rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<std::vector<rate_call> > > der_rate_functions_;
      
      // Derivative of rate functions from state 0
      std::vector<std::vector<rate_call> > der_rate_functions_0;
      
      // Derivative of rate functions from state 0 to state 1
      std::vector<rate_call> der_rate_functions_0_1;
      der_rate_functions_0_1.push_back(der_rate_func_0_1_wrt_Ca);
      der_rate_functions_0.push_back(der_rate_functions_0_1);
      
      der_rate_functions_.push_back(der_rate_functions_0);
      
      // Derivative of rate functions from state 1
      std::vector<std::vector<rate_call> > der_rate_functions_1;
      
      // Derivative of rate functions from state 1 to state 0
      std::vector<rate_call> der_rate_functions_1_0;
      der_rate_functions_1_0.push_back(der_rate_func_1_0_wrt_Ca);
      der_rate_functions_1.push_back(der_rate_functions_1_0);
      
      der_rate_functions_.push_back(der_rate_functions_1);

      // return by value
      return der_rate_functions_;
    }
    
  }

  /// Initialize static variables
  double Ca_lower_bind::_temp_scale = 1.0;
  std::vector<double> Ca_lower_bind::_params = InitCa_lower_bind::params();
  const std::map<std::string, uint> Ca_lower_bind::_param_map = InitCa_lower_bind::param_map();
  const std::vector<std::vector<uint> > Ca_lower_bind::_transition_map = InitCa_lower_bind::transition_map();
  const std::vector<std::vector<rate_call> > Ca_lower_bind::_rate_functions = InitCa_lower_bind::rate_functions();
  const std::vector<std::vector<std::vector<rate_call> > > Ca_lower_bind::_rate_derivative_functions = InitCa_lower_bind::der_rate_functions();

}

#endif



// gillstep generated Markov model code

#ifndef __CA_UPPER_BIND_H
#define __CA_UPPER_BIND_H

#include <vector>
#include <map>
#include <cmath>
#include <cassert>
#include <stdexcept>
#include <algorithm>
#include <boost/bind.hpp>

#include <gillstep/common.h>
#include <gillstep/MarkovModel.h>
#include <gillstep/GillespieStepper.h>
#include <gillstep/temperature.h>

namespace gillstep
{

  /// Markov model 
  class Ca_upper_bind : public MarkovModel
  {
      
  
  public:

    /// Default constructor
    Ca_upper_bind(GillespieStepper& gillstepper) :
        MarkovModel("Ca_upper_bind", gillstepper),
        _present_substep(0.0),
        _variables_05(1),
        _rate_05(1),
        _rate_deriviatives_05(1)
        
    {
      /// Resize field
      _variables_0.resize(1);
      _variables_1.resize(1);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["Ca"] = boost::bind(&Ca_upper_bind::_receive_Ca, this, _1);
      _variable_map["Ca"] = 0;
    }
    
    /// SubMarkov model constructor
    Ca_upper_bind(MarkovModel* parent, GillespieStepper& gillstepper) :
        MarkovModel("Ca_upper_bind", gillstepper),
        _present_substep(0.0),
        _variables_05(1),
        _rate_05(1),
        _rate_deriviatives_05(1)
    {
      // Assign parent pointer
      _parent = parent;

      /// Resize field
      _variables_0.resize(1);
      _variables_1.resize(1);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["Ca"] = boost::bind(&Ca_upper_bind::_receive_Ca, this, _1);
      _variable_map["Ca"] = 0;

      // Check if model can receive "parent"
      if (can_receive(parent->name()))
      {
        parent->connect(receive(parent->name()));
        _receive_parent = true;
      }
    
    }
    
    /// Destructor
    virtual ~Ca_upper_bind(){};
    
    /// Returns the maximal transitions from a state
    uint max_transitions() const
    {
       return 1;
    }

    /// Return the 'value' of the Markov model the state represent 
    double value() const
    {
      switch (_state)
      {
        case 1:
          return 1;
      default:
        return 0.0;
      }
    
    }

    /// Return the size of the substep. 1.0 if no substep is needed
    double substep_size(double present_substep)
    {
      assert(present_substep >= 0.0);
        
      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep)
      {
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      double next_substep = 1;
      for (uint trans = 0; trans < _transition_map[_state].size(); trans++)
      {
        if (std::fabs(_rate_deriviatives_05[trans]) > 1e-10)
            next_substep = std::min(next_substep, std::fabs(
                           _gillstepper->get_max_fractional_rate_change()*
                           (_rate_05[trans]/_rate_deriviatives_05[trans])));
      }
      
      // Update the the next substep result with the present one
      next_substep = present_substep + next_substep;

      // Return next substep. Return 1.0 if it is larger than 1.0
      return next_substep > 1.0 ? 1.0 : next_substep;
    }

    /// Evaluate rate function corresponding to passed transition
    double evaluate_rate(uint transition) const
    {
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=Ca_upper_bind::_transition_map[_state].size())
        return 0.0;

      // Evaluate the rate
      return Ca_upper_bind::_rate_functions[_state][transition](*this);
    }
    
    /// Evaluate rate function corresponding to passed state and transition
    double evaluate_rate(uint from_state, uint to_state) const
    {
      switch (from_state)
      {
        case 0:
          switch (to_state) {
            case 1:
              return Ca_upper_bind::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
        case 1:
          switch (to_state) {
            case 0:
              return Ca_upper_bind::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
      default:
        return 0.0;
      }
    }
    
    /// Evaluate rate function corresponding to passed transition for a substep
    double evaluate_substep_rate(uint transition, double present_substep,
                                 double next_substep)
    {
      assert(present_substep>=0);
      assert(next_substep>=present_substep);
      
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=Ca_upper_bind::_transition_map[_state].size())
        return 0.0;

      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      // Use tabulated values
      return _rate_05[transition] + (next_substep-_present_substep)*
        (_rate_deriviatives_05[transition]);
    }
    
    /// Change the state to the given transition
    bool change_state(int transition)
    {
      /// If forced to not change the value of this Markov Model
      if (fix_state)
        return false;
        
      /// We need to retabulate derivatives
      _retabulate_derivatives = true;
        
      const int num_transitions = Ca_upper_bind::_transition_map[_state].size();
      assert(transition<num_transitions);
        
      /// Update the state from the transition
      if (transition>=0)
        _state = Ca_upper_bind::_transition_map[_state][transition];

      /// If this is a SubMarkov model return the parents change_state value
      if (_parent)
        return _parent->change_state(-1);

      /// If this is not a SubMarkov model check if my value has change
      const bool changed = _old_value != value();
      _old_value = value();
      return changed;
    }

    /// Get temperature scale
    static inline double temp_scale()
    {
      return Ca_upper_bind::_temp_scale;
    }

    /// Get parameter
    static inline double get_parameter(uint ind)
    {
      return Ca_upper_bind::_params[ind];
    }

    /// Get parameter
    double get_parameter(const std::string& name) const
    {
      return _params[_find_parameter(name)];
    }
    
    /// Set parameter
    void set_parameter(const std::string& name, double value)
    {
      //std::cout << "Setting " << name << " to " << value << std::endl;
      _params[_find_parameter(name)] = value;
      // If the temperature scale is change update it
      if (name == "T_ref" or name == "Q10")
        update_temp_scale();
    }

    /// Get parameter names
    virtual std::vector<std::string> get_parameter_names() const
    {
      std::vector<std::string> parameter_names;

      parameter_names.push_back("Q10");
      parameter_names.push_back("T_ref");
      parameter_names.push_back("k_off");
      parameter_names.push_back("k_on");

      // Return by value
      return parameter_names;
    };

    /// Return the number of states
    inline uint num_states() const
    {
      return 2;
    }

    /// Update the temperature scale factor
    void update_temp_scale()
    {
      Ca_upper_bind::_temp_scale = pow(Ca_upper_bind::_params[0], ((get_T()-T_abs())-Ca_upper_bind::_params[1])/10);
    }

  private:

    /// Tabulate rate derivatives
    void _tabulate_rate_deriviatives()
    {
      // Local variables
      uint trans, var;

      // Update variables at present substep
      for (var=0; var < _variables_05.size(); var++)
      {
        _variables_05[var] = _variables_1[var]*_present_substep +
                             (1-_present_substep)*_variables_0[var];
        //std::cout << "Variable: " << _variables_05[var] << std::endl;
      }
        
      // Set the present variables to present interpolated values
      _present_variables = &_variables_05;

      for (trans=0; trans < _transition_map[_state].size(); trans++)
      {
        _rate_05[trans] = std::max(0.1, Ca_upper_bind::_rate_functions
                                   [_state][trans](*this));
        //std::cout << "Rate: " << _rate_05[var] << std::endl;
        _rate_deriviatives_05[trans] = 0.0;
        for (var=0; var<_variables_0.size(); var++){
          _rate_deriviatives_05[trans] += Ca_upper_bind::_rate_derivative_functions
                                         [_state][trans][var](*this)*
                                         (_variables_1[var] - _variables_05[var]);
        //std::cout << "Rate der: " << _rate_deriviatives_05[var] << std::endl;
        }
      }

      // Set the present variables to the last received values
      _present_variables = &_variables_1;

      // Set flag preventing unessesary tabulation
      _retabulate_derivatives = false;
    }

    /// Check parameter name
    static uint _find_parameter(const std::string& name)                              
    {
      std::map<std::string, uint>::const_iterator it = Ca_upper_bind::_param_map.find(name);
      if (it == Ca_upper_bind::_param_map.end())
        throw std::runtime_error("\"" + name + "\" is not a parameter in this Markov Model");                         
      return it->second;
    }

    
    /// Receive methods
    inline void _receive_Ca(double value)
    {
      _retabulate_derivatives = true;
      _variables_0[0] = _variables_1[0];
      _variables_1[0] = value;
    }
    

    /// Data members

    // Keep track of which substep we are on
    double _present_substep;

    // Variables values inside time step
    std::vector<double> _variables_05;

    /// The interpolated rate values and its derivatives wrt to the variables
    std::vector<double> _rate_05;
    std::vector<double> _rate_deriviatives_05;

      
    /// Temperature scale factor
    static double _temp_scale;

    /// An array for the parameter values
    static std::vector<double> _params;

    /// Mapping names to parameter indices
    static const std::map<std::string, uint> _param_map;
    
    /// Mapping state and transition number to state
    static const std::vector<std::vector<uint> > _transition_map;

    /// Mapping state and transition number to rate function
    static const std::vector<std::vector<rate_call> > _rate_functions;

    /// Mapping state, transition number and continuous variables to rate
    /// derivative function
    static const std::vector<std::vector<std::vector<rate_call> > > _rate_derivative_functions;
    
  };

  // Name space for hiding static initalization functions
  namespace InitCa_upper_bind
  {

    /// Define Rate functions
    
    /// Rate function between state: 0 and 1 
    double rate_func_0_1(const MarkovModel& mm)
    {
      const double value = Ca_upper_bind::temp_scale()*(Ca_upper_bind::get_parameter(3)*mm._unsafe_variable(0));
    //  std::cout << mm.name() << " tau: 0-1: " << 1/value << std::endl;
      return value;
    }
    
    
    /// Rate function between state: 1 and 0 
    double rate_func_1_0(const MarkovModel& mm)
    {
      const double value = Ca_upper_bind::temp_scale()*(Ca_upper_bind::get_parameter(2));
    //  std::cout << mm.name() << " tau: 1-0: " << 1/value << std::endl;
      return value;
    }
    
    

    /// Define derivatives of Rate functions
    
    /// Derivative of rate function between state: 0 and 1 for variable: Ca
    double der_rate_func_0_1_wrt_Ca(const MarkovModel& mm)
    {
      return Ca_upper_bind::temp_scale()*(Ca_upper_bind::get_parameter(3));
    }
    
    
    /// Derivative of rate function between state: 1 and 0 for variable: Ca
    double der_rate_func_1_0_wrt_Ca(const MarkovModel& mm)
    {
      return Ca_upper_bind::temp_scale()*(0);
    }
    
    
    /// Init params
    std::vector<double> params()
    {
      // vector which will be returned by value
      std::vector<double> params_;
      params_.push_back(1.000000);
      params_.push_back(37.000000);
      params_.push_back(1.000000);
      params_.push_back(0.500000);

      // return by value
      return params_;
    }

    /// Init param_map
    const std::map<std::string, uint> param_map()
    {
      // map which will be returned by value
      std::map<std::string, uint> param_map_;
      param_map_["Q10"] = 0;
      param_map_["T_ref"] = 1;
      param_map_["k_off"] = 2;
      param_map_["k_on"] = 3;

      // return by value
      return param_map_;
    }

    /// Init transition_map
    const std::vector<std::vector<uint> > transition_map()
    {
      // vector which will be returned by value
      std::vector<std::vector<uint> > transition_map_;
      
      // Transition map from state 0
      std::vector<uint> transition_map_0;
      transition_map_0.push_back(1);
      transition_map_.push_back(transition_map_0);
      
      // Transition map from state 1
      std::vector<uint> transition_map_1;
      transition_map_1.push_back(0);
      transition_map_.push_back(transition_map_1);

      // return by value
      return transition_map_;
    }

    /// Init rate_functions
    const std::vector<std::vector<rate_call> > rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<rate_call> > rate_functions_;
      
      // Rate functions from state 0
      std::vector<rate_call> rate_functions_0;
      rate_functions_0.push_back(rate_func_0_1);
      rate_functions_.push_back(rate_functions_0);
      
      // Rate functions from state 1
      std::vector<rate_call> rate_functions_1;
      rate_functions_1.push_back(rate_func_1_0);
      rate_functions_.push_back(rate_functions_1);

      // return by value
      return rate_functions_;
    }
    
    /// Init derivative of rate_functions
    const std::vector<std::vector<std::vector<rate_call> > > der_rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<std::vector<rate_call> > > der_rate_functions_;
      
      // Derivative of rate functions from state 0
      std::vector<std::vector<rate_call> > der_rate_functions_0;
      
      // Derivative of rate functions from state 0 to state 1
      std::vector<rate_call> der_rate_functions_0_1;
      der_rate_functions_0_1.push_back(der_rate_func_0_1_wrt_Ca);
      der_rate_functions_0.push_back(der_rate_functions_0_1);
      
      der_rate_functions_.push_back(der_rate_functions_0);
      
      // Derivative of rate functions from state 1
      std::vector<std::vector<rate_call> > der_rate_functions_1;
      
      // Derivative of rate functions from state 1 to state 0
      std::vector<rate_call> der_rate_functions_1_0;
      der_rate_functions_1_0.push_back(der_rate_func_1_0_wrt_Ca);
      der_rate_functions_1.push_back(der_rate_functions_1_0);
      
      der_rate_functions_.push_back(der_rate_functions_1);

      // return by value
      return der_rate_functions_;
    }
    
  }

  /// Initialize static variables
  double Ca_upper_bind::_temp_scale = 1.0;
  std::vector<double> Ca_upper_bind::_params = InitCa_upper_bind::params();
  const std::map<std::string, uint> Ca_upper_bind::_param_map = InitCa_upper_bind::param_map();
  const std::vector<std::vector<uint> > Ca_upper_bind::_transition_map = InitCa_upper_bind::transition_map();
  const std::vector<std::vector<rate_call> > Ca_upper_bind::_rate_functions = InitCa_upper_bind::rate_functions();
  const std::vector<std::vector<std::vector<rate_call> > > Ca_upper_bind::_rate_derivative_functions = InitCa_upper_bind::der_rate_functions();

}

#endif



// gillstep generated Markov model code

#ifndef __CHANNELGATE2_H
#define __CHANNELGATE2_H

#include <vector>
#include <map>
#include <cmath>
#include <cassert>
#include <stdexcept>
#include <algorithm>
#include <boost/bind.hpp>

#include <gillstep/common.h>
#include <gillstep/MarkovModel.h>
#include <gillstep/GillespieStepper.h>
#include <gillstep/temperature.h>

namespace gillstep
{

  /// Markov model 
  class ChannelGate2 : public MarkovModel
  {
    
    /// Sub models friends
    
  
  public:

    /// Default constructor
    ChannelGate2(GillespieStepper& gillstepper) :
        MarkovModel("ChannelGate2", gillstepper),
        _present_substep(0.0),
        _variables_05(0),
        _rate_05(1),
        _rate_deriviatives_05(1)
        , sub_Ca_lower_bind_1(this, gillstepper), sub_Ca_upper_bind_1(this, gillstepper)
    {
      /// Resize field
      _variables_0.resize(0);
      _variables_1.resize(0);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;
      _submodels.push_back(&sub_Ca_lower_bind_1);
      _submodels.push_back(&sub_Ca_upper_bind_1);

    }
    
    /// SubMarkov model constructor
    ChannelGate2(MarkovModel* parent, GillespieStepper& gillstepper) :
        MarkovModel("ChannelGate2", gillstepper),
        _present_substep(0.0),
        _variables_05(0),
        _rate_05(1),
        _rate_deriviatives_05(1), sub_Ca_lower_bind_1(this, gillstepper), sub_Ca_upper_bind_1(this, gillstepper)
    {
      // Assign parent pointer
      _parent = parent;

      /// Resize field
      _variables_0.resize(0);
      _variables_1.resize(0);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;
      _submodels.push_back(&sub_Ca_lower_bind_1);
      _submodels.push_back(&sub_Ca_upper_bind_1);


      // Check if model can receive "parent"
      if (can_receive(parent->name()))
      {
        parent->connect(receive(parent->name()));
        _receive_parent = true;
      }
    
    }
    
    /// Destructor
    virtual ~ChannelGate2(){};
    
    /// Returns the maximal transitions from a state
    uint max_transitions() const
    {
       return 1;
    }

    /// Return the 'value' of the Markov model the state represent 
    double value() const
    {
      switch (_state)
      {
        case 1:
          return 1;
      default:
        return 0.0;
      }
    
    }

    /// Return the size of the substep. 1.0 if no substep is needed
    double substep_size(double present_substep)
    {
      assert(present_substep >= 0.0);
        
      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep)
      {
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      double next_substep = 1;
      for (uint trans = 0; trans < _transition_map[_state].size(); trans++)
      {
        if (std::fabs(_rate_deriviatives_05[trans]) > 1e-10)
            next_substep = std::min(next_substep, std::fabs(
                           _gillstepper->get_max_fractional_rate_change()*
                           (_rate_05[trans]/_rate_deriviatives_05[trans])));
      }
      
      // Update the the next substep result with the present one
      next_substep = present_substep + next_substep;

      // Return next substep. Return 1.0 if it is larger than 1.0
      return next_substep > 1.0 ? 1.0 : next_substep;
    }

    /// Evaluate rate function corresponding to passed transition
    double evaluate_rate(uint transition) const
    {
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=ChannelGate2::_transition_map[_state].size())
        return 0.0;

      // Evaluate the rate
      return ChannelGate2::_rate_functions[_state][transition](*this);
    }
    
    /// Evaluate rate function corresponding to passed state and transition
    double evaluate_rate(uint from_state, uint to_state) const
    {
      switch (from_state)
      {
        case 0:
          switch (to_state) {
            case 1:
              return ChannelGate2::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
        case 1:
          switch (to_state) {
            case 0:
              return ChannelGate2::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
      default:
        return 0.0;
      }
    }
    
    /// Evaluate rate function corresponding to passed transition for a substep
    double evaluate_substep_rate(uint transition, double present_substep,
                                 double next_substep)
    {
      assert(present_substep>=0);
      assert(next_substep>=present_substep);
      
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=ChannelGate2::_transition_map[_state].size())
        return 0.0;

      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      // Use tabulated values
      return _rate_05[transition] + (next_substep-_present_substep)*
        (_rate_deriviatives_05[transition]);
    }
    
    /// Change the state to the given transition
    bool change_state(int transition)
    {
      /// If forced to not change the value of this Markov Model
      if (fix_state)
        return false;
        
      /// We need to retabulate derivatives
      _retabulate_derivatives = true;
        
      const int num_transitions = ChannelGate2::_transition_map[_state].size();
      assert(transition<num_transitions);
        
      /// Update the state from the transition
      if (transition>=0)
        _state = ChannelGate2::_transition_map[_state][transition];

      /// If this is a SubMarkov model return the parents change_state value
      if (_parent)
        return _parent->change_state(-1);

      /// If this is not a SubMarkov model check if my value has change
      const bool changed = _old_value != value();
      _old_value = value();
      return changed;
    }

    /// Get temperature scale
    static inline double temp_scale()
    {
      return ChannelGate2::_temp_scale;
    }

    /// Get parameter
    static inline double get_parameter(uint ind)
    {
      return ChannelGate2::_params[ind];
    }

    /// Get parameter
    double get_parameter(const std::string& name) const
    {
      return _params[_find_parameter(name)];
    }
    
    /// Set parameter
    void set_parameter(const std::string& name, double value)
    {
      //std::cout << "Setting " << name << " to " << value << std::endl;
      _params[_find_parameter(name)] = value;
      // If the temperature scale is change update it
      if (name == "T_ref" or name == "Q10")
        update_temp_scale();
    }

    /// Get parameter names
    virtual std::vector<std::string> get_parameter_names() const
    {
      std::vector<std::string> parameter_names;

      parameter_names.push_back("Q10");
      parameter_names.push_back("T_ref");
      parameter_names.push_back("delta_E_lower");
      parameter_names.push_back("delta_E_upper");
      parameter_names.push_back("k_off");
      parameter_names.push_back("k_on");

      // Return by value
      return parameter_names;
    };

    /// Return the number of states
    inline uint num_states() const
    {
      return 2;
    }

    /// Update the temperature scale factor
    void update_temp_scale()
    {
      ChannelGate2::_temp_scale = pow(ChannelGate2::_params[0], ((get_T()-T_abs())-ChannelGate2::_params[1])/10);
    }

  private:

    /// Tabulate rate derivatives
    void _tabulate_rate_deriviatives()
    {
      // Local variables
      uint trans, var;

      // Update variables at present substep
      for (var=0; var < _variables_05.size(); var++)
      {
        _variables_05[var] = _variables_1[var]*_present_substep +
                             (1-_present_substep)*_variables_0[var];
        //std::cout << "Variable: " << _variables_05[var] << std::endl;
      }
        
      // Set the present variables to present interpolated values
      _present_variables = &_variables_05;

      for (trans=0; trans < _transition_map[_state].size(); trans++)
      {
        _rate_05[trans] = std::max(0.1, ChannelGate2::_rate_functions
                                   [_state][trans](*this));
        //std::cout << "Rate: " << _rate_05[var] << std::endl;
        _rate_deriviatives_05[trans] = 0.0;
        for (var=0; var<_variables_0.size(); var++){
          _rate_deriviatives_05[trans] += ChannelGate2::_rate_derivative_functions
                                         [_state][trans][var](*this)*
                                         (_variables_1[var] - _variables_05[var]);
        //std::cout << "Rate der: " << _rate_deriviatives_05[var] << std::endl;
        }
      }

      // Set the present variables to the last received values
      _present_variables = &_variables_1;

      // Set flag preventing unessesary tabulation
      _retabulate_derivatives = false;
    }

    /// Check parameter name
    static uint _find_parameter(const std::string& name)                              
    {
      std::map<std::string, uint>::const_iterator it = ChannelGate2::_param_map.find(name);
      if (it == ChannelGate2::_param_map.end())
        throw std::runtime_error("\"" + name + "\" is not a parameter in this Markov Model");                         
      return it->second;
    }



    /// Data members

    // Keep track of which substep we are on
    double _present_substep;

    // Variables values inside time step
    std::vector<double> _variables_05;

    /// The interpolated rate values and its derivatives wrt to the variables
    std::vector<double> _rate_05;
    std::vector<double> _rate_deriviatives_05;

    
    /// Sub models
    Ca_lower_bind sub_Ca_lower_bind_1;
    Ca_upper_bind sub_Ca_upper_bind_1;
    
    /// Temperature scale factor
    static double _temp_scale;

    /// An array for the parameter values
    static std::vector<double> _params;

    /// Mapping names to parameter indices
    static const std::map<std::string, uint> _param_map;
    
    /// Mapping state and transition number to state
    static const std::vector<std::vector<uint> > _transition_map;

    /// Mapping state and transition number to rate function
    static const std::vector<std::vector<rate_call> > _rate_functions;

    /// Mapping state, transition number and continuous variables to rate
    /// derivative function
    static const std::vector<std::vector<std::vector<rate_call> > > _rate_derivative_functions;
    
  };

  // Name space for hiding static initalization functions
  namespace InitChannelGate2
  {

    /// Define Rate functions
    
    /// Rate function between state: 0 and 1 
    double rate_func_0_1(const MarkovModel& mm)
    {
      const double value = ChannelGate2::temp_scale()*(ChannelGate2::get_parameter(5)*std::exp(ChannelGate2::get_parameter(2)*mm.submodel(0).value())*std::exp(ChannelGate2::get_parameter(3)*mm.submodel(1).value()));
    //  std::cout << mm.name() << " tau: 0-1: " << 1/value << std::endl;
      return value;
    }
    
    
    /// Rate function between state: 1 and 0 
    double rate_func_1_0(const MarkovModel& mm)
    {
      const double value = ChannelGate2::temp_scale()*(ChannelGate2::get_parameter(4)*std::exp(-ChannelGate2::get_parameter(2)*mm.submodel(0).value())*std::exp(-ChannelGate2::get_parameter(3)*mm.submodel(1).value()));
    //  std::cout << mm.name() << " tau: 1-0: " << 1/value << std::endl;
      return value;
    }
    
    

    /// Define derivatives of Rate functions
    
    /// Init params
    std::vector<double> params()
    {
      // vector which will be returned by value
      std::vector<double> params_;
      params_.push_back(1.000000);
      params_.push_back(37.000000);
      params_.push_back(-0.500000);
      params_.push_back(0.500000);
      params_.push_back(1.000000);
      params_.push_back(1.000000);

      // return by value
      return params_;
    }

    /// Init param_map
    const std::map<std::string, uint> param_map()
    {
      // map which will be returned by value
      std::map<std::string, uint> param_map_;
      param_map_["Q10"] = 0;
      param_map_["T_ref"] = 1;
      param_map_["delta_E_lower"] = 2;
      param_map_["delta_E_upper"] = 3;
      param_map_["k_off"] = 4;
      param_map_["k_on"] = 5;

      // return by value
      return param_map_;
    }

    /// Init transition_map
    const std::vector<std::vector<uint> > transition_map()
    {
      // vector which will be returned by value
      std::vector<std::vector<uint> > transition_map_;
      
      // Transition map from state 0
      std::vector<uint> transition_map_0;
      transition_map_0.push_back(1);
      transition_map_.push_back(transition_map_0);
      
      // Transition map from state 1
      std::vector<uint> transition_map_1;
      transition_map_1.push_back(0);
      transition_map_.push_back(transition_map_1);

      // return by value
      return transition_map_;
    }

    /// Init rate_functions
    const std::vector<std::vector<rate_call> > rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<rate_call> > rate_functions_;
      
      // Rate functions from state 0
      std::vector<rate_call> rate_functions_0;
      rate_functions_0.push_back(rate_func_0_1);
      rate_functions_.push_back(rate_functions_0);
      
      // Rate functions from state 1
      std::vector<rate_call> rate_functions_1;
      rate_functions_1.push_back(rate_func_1_0);
      rate_functions_.push_back(rate_functions_1);

      // return by value
      return rate_functions_;
    }
    
    /// Init derivative of rate_functions
    const std::vector<std::vector<std::vector<rate_call> > > der_rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<std::vector<rate_call> > > der_rate_functions_;
      
      // Derivative of rate functions from state 0
      std::vector<std::vector<rate_call> > der_rate_functions_0;
      
      // Derivative of rate functions from state 0 to state 1
      std::vector<rate_call> der_rate_functions_0_1;
      der_rate_functions_0.push_back(der_rate_functions_0_1);
      
      der_rate_functions_.push_back(der_rate_functions_0);
      
      // Derivative of rate functions from state 1
      std::vector<std::vector<rate_call> > der_rate_functions_1;
      
      // Derivative of rate functions from state 1 to state 0
      std::vector<rate_call> der_rate_functions_1_0;
      der_rate_functions_1.push_back(der_rate_functions_1_0);
      
      der_rate_functions_.push_back(der_rate_functions_1);

      // return by value
      return der_rate_functions_;
    }
    
  }

  /// Initialize static variables
  double ChannelGate2::_temp_scale = 1.0;
  std::vector<double> ChannelGate2::_params = InitChannelGate2::params();
  const std::map<std::string, uint> ChannelGate2::_param_map = InitChannelGate2::param_map();
  const std::vector<std::vector<uint> > ChannelGate2::_transition_map = InitChannelGate2::transition_map();
  const std::vector<std::vector<rate_call> > ChannelGate2::_rate_functions = InitChannelGate2::rate_functions();
  const std::vector<std::vector<std::vector<rate_call> > > ChannelGate2::_rate_derivative_functions = InitChannelGate2::der_rate_functions();

}

#endif
