"""
A small test example for channel flow
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-16 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Domains
mesh = Mesh("channel-mesh")
lower = MeshDomain("lower", mesh, 2)
upper  = MeshDomain("upper", mesh, 3)

# Declare a stochastic MarkovModel which will controll the Boundary
channel_gate = DiscreteMarkovModel("ChannelGate2")

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), (20, 10), 1, sigma=sigma)

# Connect controller with Boundary
Connection(channel_gate, channel)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma*5)
ScalarFieldSpecies(upper, Ca, 10., sigma)

# Declare sub Markov model which will represent discrete binding of Ca
# ions to the two sides of the channel
Ca_lower_bind = DiscreteMarkovSubModel(channel_gate, "Ca_lower_bind")
Ca_upper_bind = DiscreteMarkovSubModel(channel_gate, "Ca_upper_bind")

# Connect the Ca field to the sub markov model. Need to provide a particular
# boundary, here channel, for what value of the Ca will be provided to the
# markov model
Connection(lower.Ca.channel, Ca_lower_bind)
Connection(upper.Ca.channel, Ca_upper_bind)

# Declare flux on Boundary
DiffusionFlux(channel, Ca, channel.sigma)

