"""
A small test example for channel flow
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-16 -- 2011-10-04"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Declare domains
mesh = Mesh("channel-mesh")
lower = MeshDomain("lower", mesh, 2)
upper  = MeshDomain("upper", mesh, 3)

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), (20, 10), 1, sigma=0.2)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma*5)
ScalarFieldSpecies(upper, Ca, 10, sigma)

cond = Conditional(Le(t, tstop/2), t*channel.sigma, \
                   Conditional(Le(t, 3*tstop/4), (tstop-t)*channel.sigma, (tstop-t)**2))

# Declare flux on the Boundary
Flux("TimeDep", channel, Ca, cond)

