# Import SubCell
from subcell import *

set_log_level(DEBUG)

# Default parameters all accessible from the command prompt
default_params = ParameterDict(
    tstop = RangeCheck(30, gt=0),
    dt_max_times = [0, 5],
    dt_max_values = [1, .5],
    num_runs = RangeCheck(1, ge=1),
    )

def main(params):
    """The main function which load and runs the model"""

    # Create the timestepper
    timestepper = TimeStepper("channel-timedep")
    p = timestepper.params
    
    # Set casename
    p.io.casename = "channel-timedep"
    p.io.use_logfile = True
    
    # Set parameters
    p.diffreact.io.field_species.lowerCa_upperCa.plot = True
    
    # Solver parameters
    p.diffreact.newton_solver.absolute_tolerance = 1e-6
    p.diffreact.newton_solver.relative_tolerance = 1e-6
    
    # Time stepping parameters
    p.time.tstop = params.tstop
    p.time.dt_max_times = params.dt_max_times
    p.time.dt_max_values = params.dt_max_values
    p.time.num_runs = params.num_runs

    # Print Parameters
    #print p
    
    # Just do it
    timestepper.run()
    
if __name__ == "__main__":
    default_params.parse_args()
    main(default_params)
