"""
A small test example for channel flow
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-16 -- 2011-11-01"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Declare domains
mesh = Mesh("channel-mesh")
lower = MeshDomain("lower", mesh, 2)
upper  = MeshDomain("upper", mesh, 3)

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), (20, 10), dx=1, sigma=sigma)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma*5)
ScalarFieldSpecies(upper, Ca, 10, sigma)

# Declare flux on the Boundary
DiffusionFlux(channel, Ca, channel.sigma)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.], True)

# Declare connection between channel controller and the channel
Connection(channel_ctrl, channel)
