# Import SubCell
from subcell import *

# Default parameters all accessible from the command prompt
default_params = ParameterDict(
    tstop = RangeCheck(30, gt=0),
    dt_max = RangeCheck(0.5, gt=0, le=5.),
    num_runs = RangeCheck(1, ge=1),
    stoch = ParameterDict(
        k_on = RangeCheck(0.5, gt=0.),
        k_off = RangeCheck(0.5, gt=0.),
        ),
    )

def main(params):
    """The main function which load and runs the model"""

    # Create the timestepper
    timestepper = TimeStepper("complex-channel-stoch")
    p = timestepper.params
    
    # Set casename
    p.io.casename = "complex-channel-stoch"
    p.io.use_logfile = True
    
    # Set parameters
    plot_dict = dict(plot=True, vmax=200)
    p.diffreact.io.field_species.\
        lower_leftCa_upper_leftCa_lower_rightCa_upper_rightCa.update(plot_dict)

    p.stochastic.collect_values = True
    p.stochastic.models.ChannelGate.update(params.stoch)

    # Use numbers in UFL forms instead of Constant, will speed up calculations
    # but will trigger recomplation of forms each time the parameters changes
    p.diffreact.params_to_numbers = True

    # Track conservation
    p.diffreact.io.conservation = ["Ca"] 
    
    # Solver parameters
    p.diffreact.newton_solver.absolute_tolerance = 1e-6
    p.diffreact.newton_solver.relative_tolerance = 1e-6
    
    # Time stepping parameters
    p.time.tstop = params.tstop
    p.time.dt_max = params.dt_max
    p.time.num_runs = params.num_runs

    # Print Parameters
    print p
    
    # Just do it
    timestepper.run()
    
if __name__ == "__main__":
    default_params.parse_args()
    main(default_params)

