"""
A test example for a complex channel flow
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-19 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameters
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7),
                dx=RangeCheck(1., gt=0), scale=100, pump_speed=200)

# Declare domains
mesh = Mesh("channel-4-mesh")
lower_left = MeshDomain("lower_left", mesh, 2)
upper_left = MeshDomain("upper_left", mesh, 3)
lower_right = MeshDomain("lower_right", mesh, 4)
upper_right = MeshDomain("upper_right", mesh, 5)

# Declare boundaries
left_channel = MeshBoundary("left_channel", (upper_left, lower_left), (20, 10), dx)
lower_channel = MeshBoundary("lower_channel", (lower_left, lower_right), (50, 60), dx)
right_channel = MeshBoundary("right_channel", (lower_right, upper_right), (30, 40), dx)
upper_channel = MeshBoundary("upper_channel", (upper_right, upper_left), (80, 70), dx)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.], True)

# Connect controller with Boundaries
Connection(channel_ctrl, left_channel)
Connection(channel_ctrl, lower_channel)
Connection(channel_ctrl, upper_channel)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower_left, Ca, 50, sigma*50)
ScalarFieldSpecies(upper_left, Ca, 100, sigma*50)
ScalarFieldSpecies(lower_right, Ca, 10, sigma*5)
ScalarFieldSpecies(upper_right, Ca, 200., sigma*5)

# Declare fluxes on Boundaries
DiffusionFlux(left_channel, Ca, scale*sigma)
DiffusionFlux(lower_channel, Ca, scale*sigma)
Flux("Pump", right_channel, Ca, pump_speed/(1+(10/lower_right.Ca)**2))
DiffusionFlux(upper_channel, Ca, scale*sigma)
