# LCC markov model
# 
# Model and rated taken from:
# Greenstein, J. L. & Winslow, R. L.
# An integrative model of the cardiac ventricular myocyte incorporating local control of Ca$^2+$ release.
# Biophys. J., 2002, 83, 2918-2945
# 
# Based on the model from:
# 
# Jafri, M. S.; Rice, J. J. & Winslow, R. L.
# Cardiac Ca$^2+$ dynamics: the roles of ryanodine receptor adaptation
# and sarcoplasmic reticulum load.
# Biophys. J., 1998, 74, 1149-68
# 
# The rates are all fitted for 37degC
    
# Declarations
Parameters(f      = 0.85,
           g      = 2.0,
           a      = 2.0,
           b      = 1.9356,
           gamma0 = 4.4e-4,
           omega  = 0.02158)

Variables("V", "Ca")

Q10(1.8, 37)

# Rate functions
alpha = 2*exp(0.012*(V-35))
beta = 0.0882*exp(-0.05*(V-35))
gamma = gamma0*Ca

# Voltage dependent rates
for i in xrange(4):
    Rates[i,i+1]   = (4-i)*alpha
    Rates[i+1,i]   = (i+1)*beta
    Rates[i+6,i+7] = (4-i)*alpha*a
    Rates[i+7,i+6] = (i+1)*beta/b

# Calcium inactivation
for i in xrange(5):
    Rates[i,i+6]   = gamma*a**i
    Rates[i+6,i]   = omega/b**i

# Constant opening and closing rates
Rates[4,5] = f
Rates[5,4] = g

# State values
StateValues[5] = load_submodel("VoltageGate").value
