# A model model of allosteric activation of RyR
# When a neigboring unit is open and when a Ca is bound to the RyR the
# probability to open increase. When a Mg is bound the probability decreases.
# 

# Hard wired Parameters determining the number of neighboring RyR and the
# number of ions possible to bind

# Define arguments that can be altered when compiling the model
# FIXME: Not working from command line interface...
# FIXME: only via load_{sub}_model
arguments(Num_ion_binding=4, Num_neigboring_RyRs=4)

assert(Num_ion_binding >= 1 and Num_ion_binding <=4)
assert(Num_neigboring_RyRs >= 1 and Num_neigboring_RyRs <=4)

# Parameters
Parameters(k_on_RyR=1.0, k_off_RyR=0.1, delta_E_coop=0.8)

# Externals
Variables(*["RyR_%d"%i for i in xrange(Num_neigboring_RyRs)], discrete=True)

# Load ion binding models
cyt_ion_binders = load_submodel("Ca_Mg_cyt_bind", Num_ion_binding)
cyt_ion = cyt_ion_binders[0]

# Load ion binding models
lum_ion_binders = load_submodel("Ca_Mg_lum_bind", Num_ion_binding)
lum_ion = lum_ion_binders[0]

# Function returning a Piecewise function which is either 1 or 0
# if an ion is bound or not
def Ca_bound(value):
    return Conditional(Eq(value, 1), 1, 0)

def Mg_bound(value):
    return Conditional(Eq(value, 2), 1, 0)

def add(arg0, arg1):
    return arg0 + arg1

# Variables keeping track of the number of bound ions
N_Mg_cyt = reduce(add, [Mg_bound(ion_binder.value) \
                        for ion_binder in cyt_ion_binders])
N_Ca_cyt = reduce(add, [Ca_bound(ion_binder.value) \
                        for ion_binder in cyt_ion_binders])
N_Mg_lum = reduce(add, [Mg_bound(ion_binder.value) \
                        for ion_binder in lum_ion_binders])
N_Ca_lum = reduce(add, [Ca_bound(ion_binder.value) \
                        for ion_binder in lum_ion_binders])

# Variable keeping track of active neigbors
N_open_RyRs = eval(" + ".join("RyR_%d"%i for i in xrange(Num_neigboring_RyRs)))

# Define the rates
Rates[0,1] = k_on_RyR*exp(delta_E_coop/2)**N_open_RyRs\
             *exp(cyt_ion.delta_E_Ca)**(N_Ca_cyt)\
             *exp(cyt_ion.delta_E_Mg)**(N_Mg_cyt)\
             *exp(lum_ion.delta_E_Ca/2)**(N_Ca_lum)\
             *exp(lum_ion.delta_E_Mg)**(N_Mg_lum)

Rates[1,0] = k_off_RyR*exp(delta_E_coop/2)**(-(N_open_RyRs))\
             *exp(lum_ion.delta_E_Ca/2)**(-(N_Ca_lum))#\
             #*exp(delta_E_Ca_cyt/2)**(-(N_Ca_cyt))\
             #*exp(delta_E_Mg_cyt/2)**(-(N_Mg_cyt))\
             #*exp(delta_E_Mg_lum/2)**(-(N_Mg_lum))

# Set the value of the open state
StateValues[1] = 1
