"""
A small test example for channel flow together with electro diffusion
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-12-01 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameters
ModelParameters(c0=RangeCheck(1., ge=0.1, le=1.0e7),
                sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Declare domains
mesh = Mesh("channel-mesh")
lower = MeshDomain("lower", mesh, 2, phi0=-2.2, kappa=1.0)
upper = MeshDomain("upper", mesh, 3)

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), (20, 10), 1, sigma=sigma)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.], True)

# Connect controller with Boundary
Connection(channel_ctrl, channel)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma*5)
ScalarFieldSpecies(upper, Ca, 100., sigma)

# Add a static electric field to the lower domain
StaticElectricPotential(lower, lower.phi0*exp(-lower.kappa*lower.z_node))

# Declare flux on Boundary
DiffusionFlux(channel, Ca, channel.sigma)
