"""
A small test example for channel flow with reactions
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-26 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Domains
mesh = Mesh("channel-mesh")
lower = MeshDomain("lower", mesh, 2)
upper = MeshDomain("upper", mesh, 3)

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), (20, 10), 1, sigma=sigma)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.], True)

# Connect controller with Boundary
Connection(channel_ctrl, channel)

# Declare Species
Ca = Species("Ca", 2)
SCa = Species("SCa", 0, Ca, koff=1., tot=10., kon=1.)
MCa = Species("MCa", 0, Ca, koff=2., tot=20., kon=2.)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma)
ScalarFieldSpecies(upper, Ca, 10, sigma)
FixedScalarFieldSpecies(lower, SCa, SCa.tot/(1+SCa.koff/(SCa.kon*lower.Ca.init)))
ScalarFieldSpecies(lower, MCa, MCa.tot/(1+MCa.koff/(MCa.kon*lower.Ca.init)), sigma)

# Declare flux on Boundary
DiffusionFlux(channel, Ca, channel.sigma)

# Declare reaction on the lower domain
Reaction(lower, SCa.kon*lower.Ca*(SCa.tot-lower.SCa) - SCa.koff*(lower.SCa),
         reactants=[Ca], products=[SCa])
Reaction(lower, MCa.kon*lower.Ca*(MCa.tot-lower.MCa) - MCa.koff*(lower.MCa),
         reactants=[Ca], products=[MCa])

