# Import SubCell
from subcell import *

# Default parameters all accessible from the command prompt
default_params = ParameterDict(
    tstop = RangeCheck(30, gt=0.),
    dt_max = RangeCheck(0.5, gt=0., le=5.),
    num_runs = RangeCheck(2, ge=1),
    dt_min = RangeCheck(0.00001, gt=0, le=5.),

    reaction = ParameterDict(
        split = False, 
        ),
    )

def main(params):
    """The main function which load and runs the model"""

    # Create the timestepper
    timestepper = TimeStepper("reaction-scalar")
    p = timestepper.params
    
    # Set casename
    p.io.casename = "reaction-scalar"
    p.io.use_logfile = True
    
    # Set parameters
    p.diffreact.io.field_species.lowerCa_upperCa.plot = True
    p.diffreact.io.conservation = ["Ca"] 
    p.diffreact.operator_split_reactions = params.reaction.split
    
    # Solver parameters
    p.diffreact.newton_solver.absolute_tolerance = 1e-6
    p.diffreact.newton_solver.relative_tolerance = 1e-6
    
    # Time stepping parameters
    p.time.tstop = params.tstop
    p.time.dt_max = params.dt_max
    p.time.num_runs = params.num_runs
    p.time.dt_min = params.dt_min

    # Print Parameters
    print p
    
    # Just do it
    timestepper.run()
    
if __name__ == "__main__":
    default_params.parse_args()
    main(default_params)

