"""
A small test example for channel flow with reactions
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-26 -- 2012-07-23"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Domains
mesh = Mesh("channel-mesh")
lower = MeshDomain("lower", mesh, 2)
upper  = MeshDomain("upper", mesh, 3)

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), (20, 10), 1, sigma=sigma)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.])

# Connect controller with Boundary
Connection(channel_ctrl, channel)

# Declare Species
Ca = Species("Ca", 2)
BCa = Species("BCa", 0, koff=1., tot=10., kon=1.)

# Declare domain specific species
ScalarFieldSpecies(upper, Ca, 10., sigma)
ScalarFieldSpecies(lower, Ca, 1., sigma*5)

# Scalar valued buffer
ScalarSpecies(upper, BCa, BCa.tot/(1+BCa.koff/(BCa.kon*upper.Ca.init)))

# Declare flux on Boundary
DiffusionFlux(channel, Ca, channel.sigma)

# Declare reaction on the upper domain
Reaction(upper, BCa.kon*upper.Ca*(BCa.tot-upper.BCa)- BCa.koff*(upper.BCa),
         reactants=[Ca], products=[BCa])
