"""
A small test example for channel flow with reactions
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-26 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7))

# Domains
mesh = Mesh("channel-mesh-0")
lower = MeshDomain("lower", mesh, 2)
upper  = VolumeDomain("upper", 1000.)

# Declare a value clamp representing a Voltage clamp
V = ValueClamp("V", [0.,10.,25.], [0.,1.,0.])

# Declare a stochastic MarkovModel which will controll the Boundary
channel_gate = DiscreteMarkovModel("ChannelGate")

# Connect the Voltage clamp with the MarkovModel
Connection(V, channel_gate)

# Declare a boundary on the mesh with its own parameters, sigma and dx
channel = MeshBoundary("channel", (upper, lower), 10, 1, sigma=sigma)

# Connect controller with Boundary
Connection(channel_gate, channel)

# Declare Species
Ca = Species("Ca", 2)
BCa = Species("BCa", 0, Ca, koff=1., tot=10., kon=1.)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 1., sigma)
ScalarFieldSpecies(lower, BCa, BCa.tot/(1+BCa.koff/(BCa.kon*lower.Ca.init)))
ScalarSpecies(upper, Ca, 10.)

# Declare flux on Boundary
DiffusionFlux(channel, Ca, channel.sigma)

# Declare reaction on the lower domain
Reaction(lower, BCa.kon*lower.Ca*(BCa.tot-lower.BCa)- BCa.koff*(lower.BCa),
         reactants=[Ca], products=[BCa])
