"""
A small test example for channel flow
"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-16 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

# Parameter
ModelParameters(sigma=RangeCheck(2., ge=0.1, le=1.0e7),
                dx=RangeCheck(1., gt=0))

# Declare domains
mesh = Mesh("tube-mesh")
lower = MeshDomain("lower", mesh, 1)
middle = MeshDomain("middle", mesh, 2)
upper  = MeshDomain("upper", mesh, 3)

top, bottom = InfiniteDomains("top", "bottom")

# Declare two boundaries on the mesh 
lower_channel = MeshBoundary("lower_channel", (lower, bottom), 10, dx, sigma=sigma)
upper_channel = MeshBoundary("upper_channel", (top, upper), 20, dx, sigma=sigma)

# Declare Species
Ca = Species("Ca", 2)

# Declare domain specific species
ScalarFieldSpecies(lower, Ca, 5., sigma)
ScalarFieldSpecies(middle, Ca, 10., sigma*5)
ScalarFieldSpecies(upper, Ca, 20, sigma)
ConstSpecies(top, Ca, 30)
ConstSpecies(bottom, Ca, 1)

# Declare flux on the Boundary
DiffusionFlux(lower_channel, Ca, lower_channel.sigma)
DiffusionFlux(upper_channel, Ca, upper_channel.sigma)

# Boundary controller
channel_ctrl = ValueClamp("channel_ctrl", [0.,5.,25.], [0.,1.,0.], True)

# Declare connection between channel controller and the channel
Connection(channel_ctrl, upper_channel)
Connection(channel_ctrl, lower_channel)
