"""
A declarative language and runner for subcellular diffusion reaction
problems.

Running a subcell model:
========================

The C{subcell} module includes functions and classes for running and
analyze data.

Example:
--------

>>> from subcell import *
>>> stepper = TimeStepper("model")
>>> stepper.params.io.casename = "model"
>>> stepper.params.tstop = 25
>>> stepper.run()

Here are the model loaded by instantiating a L{TimeStepper} class. For
a full overview of the available nested parameters do:

>>> print stepper.params

Different functions for analyzing generated data is also
provided. They all require that the data file is set, using
L{set_data}:

>>> data = SubCellData("data/model")
>>> current_data = data.current("channel", "Na", 2)

The name of the data file corresponds to the name of the C{casename},
see above. See L{SubCellData} for more data access functions.

Model language
==============

A model file is written in a declarative way and stored in C{.scm}
file. In the above example a model file with name C{model.scm} is
assumed to exist.

For documentation of the model language look in the module
L{modellanguage}.

"""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-11-07 -- 2011-05-03"
__license__  = "GNU LGPL Version 3.0 or later"

import subcell
import modellanguage
from subcell import *

__all__ = ["modellanguage"]
__all__.extend(subcell.__all__)

