__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-10-14 -- 2011-06-09"
__license__  = "GNU LGPL Version 3.0 or later"

__all__ = ["ContinuousBackend", "StochasticBackend"]

# subcell imports
from subcell.model.subcellmodel import SubCellModel
from subcell.common import *

class ContinuousBackend(object): 
    "Base class for a continuous backend"
    def __init__(self, subcellmodel):
        check_arg(subcellmodel, SubCellModel, 0)
        self.subcellmodel = subcellmodel
        self.time = lambda : 0.0

    def step(self, send_values=True):
        raise NotImplementedError, "implement method in subclass"

    def step_back(self, offset):
        raise NotImplementedError, "implement method in subclass"

    def connect(self, domain_specific_species, boundary, num, receiver):
        raise NotImplementedError, "implement method in subclass"
    
    def send(self):
        raise NotImplementedError, "implement method in subclass"

    def send_constant(self):
        raise NotImplementedError, "implement method in subclass"

    def handle_io(self):
        raise NotImplementedError, "implement method in subclass"
        
    def interactive(self):
        raise NotImplementedError, "implement method in subclass"

    def get_geometry(self):
        raise NotImplementedError, "implement method in subclass"
            
    def reset_before_single_run(self):
        raise NotImplementedError, "implement method in subclass"

    def get_data_after_single_run(self):
        raise NotImplementedError, "implement method in subclass"
        
    def reset_before_set_of_runs(self):
        raise NotImplementedError, "implement method in subclass"
    
class StochasticBackend(object):
    "Base class for a stochastic backend"
    def __init__(self, subcellmodel):
        check_arg(subcellmodel, SubCellModel, 0)
        self.subcellmodel = subcellmodel
        self.time = lambda : 0.0
    
    def init(self, t=0.0):
        raise NotImplementedError, "implement method in subclass"
    
    def step_back(self, offset):
        raise NotImplementedError, "implement method in subclass"

    def step_dt(self, dt):
        raise NotImplementedError, "implement method in subclass"

    def reset_before_set_of_runs(self):
        raise NotImplementedError, "implement method in subclass"
        
    def reset_before_single_run(self, time=0.0):
        raise NotImplementedError, "implement method in subclass"

    def get_data_after_single_run(self):
        raise NotImplementedError, "implement method in subclass"
        
    def set_minimize_dt_callback(self, func):
        raise NotImplementedError, "implement method in subclass"

    def set_end_sim_callback(self, func):
        raise NotImplementedError, "implement method in subclass"

    def init_state_values(self):
        raise NotImplementedError, "implement method in subclass"
        
