__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-10-14 -- 2011-05-16"
__license__  = "GNU LGPL Version 3.0 or later"

__all__ = ["load_diffusion_reaction_backend_class", \
           "load_stochastic_backend_class"]

# local imports
from subcell.common import *
from subcell.common.logger import *

diffusion_reaction_backends = ["dolfin"]
stochastic_backends = ["gillstep"]

def _load_backend(backend_type, backend):
    "helper function to load a backend"
    check_arg(backend_type, str, 0)
    check_arg(backend, str, 1)

    assert backend_type in ["stochastic", "diffusion_reaction"]
    
    # Check backend
    allowed_backends = globals()[backend_type+"_backends"]
    if backend not in allowed_backends:
        raise ValueError, "expected a %s backend from one of: %s"% \
              (backend_type, quote_join(allowed_backends))

    try:
        # Try loading the backend library
        backend_module_library = __import__(backend)
    except ImportError, e:
        warning("Install '%s' to be able to use this backend", backend)
        raise e
        
    # Load the subcell specific backend module
    backend_module = __import__("subcell.backends.%sbackend"%backend,
                                fromlist=["subcell.backends"])

    # Instantiate the backend object
    return getattr(backend_module, "%sBackend"%backend.capitalize())

def load_stochastic_backend_class(backend):
    "Load a discrete backend class"
    return _load_backend("stochastic", backend)

def load_diffusion_reaction_backend_class(backend):
    "Load a continuous backend class"
    return _load_backend("diffusion_reaction", backend)
