__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2011-06-09 -- 2011-08-18"
__license__  = "GNU LGPL Version 3.0 or later"

__all__ = ["common_snippet", "reaction_form", "nondirect_snippets", \
           "direct_snippets"]

nondirect_snippets = {}
nondirect_snippets["SOLVE"] = """
      /// Jacobian matrix
      arma::mat J(%(SYSTEMSIZE)s, %(SYSTEMSIZE)s);
      
      /// Solution vector
      arma::vec dx(%(SYSTEMSIZE)s);
      
      /// Resdiual vector
      arma::mat F(%(SYSTEMSIZE)s, 1);
      
      /// Value vector
      arma::vec u(%(SYSTEMSIZE)s);
      
      /// Initial value vector
      arma::vec u0(%(SYSTEMSIZE)s);

      // Data to get local values from Vector
      std::vector<uint> rows(%(SYSTEMSIZE)s);

      /// Parameter vector
      std::vector<double> params(_params);

      #pragma omp parallel for schedule(guided, 20) firstprivate(params, J, dx, F, u, u0, rows)
      for (uint i = 0; i < _size; i++)
      {
	
	/// Most recent residual and intitial residual
	double residual, residual0, relative_residual;
      
        uint newton_iteration = 0;
        bool newton_converged = false;
	
	/// Update parameter with vector parameter
	std::map<uint, std::vector<double> >::const_iterator it;
	for (it = _vector_params.begin(); it != _vector_params.end(); it++)
	  params[it->first] = it->second[i];
	
        // Update rows
        for (uint j = 0; j < %(SYSTEMSIZE)s; j++)
          rows[j] = j*_size+i;
	
        // Get values and initialize u and u0
        x.get_local(u0.memptr(), %(SYSTEMSIZE)s, &rows[0]);
	u = u0;
        
        // Compute initial F(u)
        compute_F(F, u0, u, params, dt);
        
        // Start iterations
        while (!newton_converged && newton_iteration < maxiter)
        {
          // Compute Jacobian
          if (recompute_jacobian || newton_iteration==0)
	    compute_J(J, u, params, dt);
    
          // Perform linear solve and update total number of Krylov iterations
          bool result = arma::solve(dx, J, F);
	  
          // Compute initial residual
          if (newton_iteration == 0)
	    // Compute resdiual
	    if (convergence_criterion == "residual")
	      residual = residual0 = arma::norm(F, 2);
	    else if (convergence_criterion == "incremental")
	      residual = residual0 = arma::norm(dx, 2);
	    else
	      error("Unknown Newton convergence criterion");
	  
          if (std::abs(1.0 - relaxation) < DOLFIN_EPS)
            u -= dx;
          else
            u -= relaxation*dx;
        
          // Update number of iterations
          ++newton_iteration;
        
          // FIXME: this step is not needed if residual is based on dx 
	  // FIXME: and this has converged.
	  compute_F(F, u0, u, params, dt);
        
          // Test for convergence
	  // Compute resdiual
	  if (convergence_criterion == "residual")
	    residual = arma::norm(F, 2);
	  else if (convergence_criterion == "incremental")
	    residual = arma::norm(dx, 2);
	  else
	    error("Unknown Newton convergence criterion");
	  
	  // Relative residual
	  relative_residual = residual / residual0;
	  
	  // Output iteration number and residual
	  if (report && (newton_iteration > 0) && (i == 0))
	  {
	    info("Reaction newton iteration %%d: r (abs) = %%.3e (tol = %%.3e) r (rel) = %%.3e (tol = %%.3e)",
		 newton_iteration, residual, atol, relative_residual, rtol);
	  }
	  
	  // Return true of convergence criterion is met
	  if (relative_residual < rtol || residual < atol)
	    newton_converged = true;
        }
    
        if (newton_converged)
        {
          // Put solution back into vector
          x.set(u.memptr(), %(SYSTEMSIZE)s, &rows[0]);
        }
        else
        {
          info("Last iteration before error %%d: r (abs) = %%.3e (tol = %%.3e) r (rel) = %%.3e (tol = %%.3e)",
		 newton_iteration, residual, atol, relative_residual, rtol);
	  error("Rection newton solver did not converge.");
        }
      }
"""

nondirect_snippets["PRIVATEFUNCS"] = """
    /// Compute Jacobian, J, using present values of u, and u0
    void compute_J(arma::mat& J, arma::vec& u, 
		   const std::vector<double>& params, double dt, )
    {
%(COMPUTE_JACOBIAN)s
    }

    /// Compute the residual vector, F, using present values of u, and u0
    void compute_F(arma::mat& F, arma::vec& u0, arma::vec& u,
		   const std::vector<double>& params, double dt)
    {
%(COMPUTE_F)s
    }
"""

direct_snippets = {}
direct_snippets["SOLVE"] = """
      /// Resdiual vector
      std::vector<double> F(%(SYSTEMSIZE)s);
      
      /// Increment vector
      std::vector<double> dx(%(SYSTEMSIZE)s);
      
      /// Value vector
      std::vector<double> u(%(SYSTEMSIZE)s);
      
      /// Initial value vector
      std::vector<double> u0(%(SYSTEMSIZE)s);

      // Data to get local values from Vector
      std::vector<uint> rows(%(SYSTEMSIZE)s);

      /// Parameter vector
      std::vector<double> params(_params);

      //for (int k=0; k<%(SYSTEMSIZE)s; k++)
      //info("x[%%d]: %%f", k+619, x[_size*k+619]);
#pragma omp parallel for schedule(guided, 20) firstprivate(params, dx, F, u, u0, rows)
      for (uint i = 0; i < _size; i++)
      {
	
	/// Most recent residual and intitial residual
	double residual, residual0, relative_residual;
      
        uint newton_iteration = 0;
        bool newton_converged = false;
	
	/// Update parameter with vector parameter
	std::map<uint, std::vector<double> >::const_iterator it;
	for (it = _vector_params.begin(); it != _vector_params.end(); it++)
	  params[it->first] = it->second[i];
	
        // Update rows
        for (uint j = 0; j < %(SYSTEMSIZE)s; j++)
          rows[j] = j*_size+i;
	
        // Get values and initialize u and u0
        x.get_local(&u0[0], %(SYSTEMSIZE)s, &rows[0]);
	
	for (uint j=0; j<%(SYSTEMSIZE)s;j++)
	  u[j] = u0[j];
        
        // Compute initial F(u)
        compute_F(F, u0, u, params, dt);
        
        // Start iterations
        while (!newton_converged && newton_iteration < maxiter)
        {
	  
          // Perform linear solve and update total number of Krylov iterations
	  solve_u(dx, u, F, params, dt);
	  
          // Compute initial residual
          if (newton_iteration == 0)
	    // Compute resdiual
	    if (convergence_criterion == "residual")
	      residual = residual0 = norm(F);
	    else if (convergence_criterion == "incremental")
	      residual = residual0 = norm(dx);
	    else
	      error("Unknown Newton convergence criterion");
	  
	  for (uint j=0; j<%(SYSTEMSIZE)s;j++)
	    u[j] = u[j] - relaxation*dx[j];
        
          // Update number of iterations
          ++newton_iteration;
        
          // FIXME: this step is not needed if residual is based on dx 
	  // FIXME: and this has converged.
	  compute_F(F, u0, u, params, dt);
        
          // Test for convergence
	  // Compute resdiual
	  if (convergence_criterion == "residual")
	    residual = norm(F);
	  else if (convergence_criterion == "incremental")
	    residual = norm(dx);
	  else
	    error("Unknown Newton convergence criterion");
	  
	  // Relative residual
	  relative_residual = residual / residual0;
	  
	  // Output iteration number and residual
	  if (report && (newton_iteration > 0) && (i == 0))
	  {
	    info("Reaction newton iteration %%d: r (abs) = %%.3e (tol = %%.3e) r (rel) = %%.3e (tol = %%.3e)",
		 newton_iteration, residual, atol, relative_residual, rtol);
	  }
	  
	  // Return true of convergence criterion is met
	  if (relative_residual < rtol || residual < atol)
	    newton_converged = true;
        }
	
        if (newton_converged)
        {
          // Put solution back into vector
          x.set(&u[0], %(SYSTEMSIZE)s, &rows[0]);
        }
        else
        {
          info("Last iteration before error %%d: r (abs) = %%.3e (tol = %%.3e) r (rel) = %%.3e (tol = %%.3e)",
		 newton_iteration, residual, atol, relative_residual, rtol);
	  error("Rection newton solver did not converge.");
        }
      }
"""

direct_snippets["SOLVE_ONCE"] = """
      /// Resdiual vector
      std::vector<double> F(%(SYSTEMSIZE)s);
      
      /// Increment vector
      std::vector<double> dx(%(SYSTEMSIZE)s);
      
      /// Value vector
      std::vector<double> u(%(SYSTEMSIZE)s);
      
      /// Initial value vector
      std::vector<double> u0(%(SYSTEMSIZE)s);

      // Data to get local values from Vector
      std::vector<uint> rows(%(SYSTEMSIZE)s);

      /// Parameter vector
      std::vector<double> params(_params);

      //for (int k=0; k<%(SYSTEMSIZE)s; k++)
      //info("x[%%d]: %%f", k+619, x[_size*k+619]);
#pragma omp parallel for schedule(guided, 20) firstprivate(params, dx, F, u, u0, rows)
      for (uint i = 0; i < _size; i++)
      {
	
	/// Update parameter with vector parameter
	std::map<uint, std::vector<double> >::const_iterator it;
	for (it = _vector_params.begin(); it != _vector_params.end(); it++)
	  params[it->first] = it->second[i];
	
        // Update rows
        for (uint j = 0; j < %(SYSTEMSIZE)s; j++)
          rows[j] = j*_size+i;
	
        // Get values and initialize u and u0
        x.get_local(&u[0], %(SYSTEMSIZE)s, &rows[0]);
        x0.get_local(&u0[0], %(SYSTEMSIZE)s, &rows[0]);
	
        // Compute initial F(u)
        compute_F(F, u0, u, params, dt);
        
        // Perform linear solve
        solve_u(dx, u, F, params, dt);
	  
        for (uint j=0; j<%(SYSTEMSIZE)s;j++)
          u[j] = u[j] - relaxation*dx[j];
        
        // Put solution back into vector
        x.set(&u[0], %(SYSTEMSIZE)s, &rows[0]);
      }
"""

direct_snippets["PRIVATEFUNCS"] = """
    /// Solve the Linear system by a symbolically inversed Jacobian matrix
    void solve_u(std::vector<double>& dx, const std::vector<double>& u, 
		 const std::vector<double>& F, const std::vector<double>& params,
		 double dt) const
    {
%(SOLVE_U)s
    }
    
    /// Compute the residual vector, F, using present values of u, and u0
    void compute_F(std::vector<double>& F, const std::vector<double>& u0, 
		   const std::vector<double>& u, const std::vector<double>& params,
		   double dt) const
    {
%(COMPUTE_F)s
    }
    
    /// Return the L2 norm of the vector
    double norm(const std::vector<double>& u) const
    {
      double _norm = 0;
      for (uint i=0; i<%(SYSTEMSIZE)s; i++)
	_norm += u[i]*u[i];
      return std::sqrt(_norm);
    }
"""

common_snippet = """
  /// This class defines a Newton solver for equations of the form F(u) = 0.
  class ReactionNewtonSolver : public Variable
  {
  public:

    /// Default parameter values
    static dolfin::Parameters default_parameters()
    {
      Parameters p("newton_solver");
      
      p.add("recompute_jacobian",      true);
      p.add("num_threads",             1, 1, 20);
      p.add("maximum_iterations",      20);
      p.add("relative_tolerance",      1e-10);
      p.add("absolute_tolerance",      1e-10);
      p.add("convergence_criterion",   "residual");
      p.add("relaxation_parameter",    1.0, 0., 1.);
      p.add("report",                  true);
      
      return p;
    }

    /// Create nonlinear solver
    ReactionNewtonSolver(uint size) : Variable("Reaction newton solver", "unamed"), 
				      _size(size), _params(%(NUMPARAMS)s, 0.0), 
				      _param_map()
    {
      // Set default parameters
      parameters = default_parameters();
      
      // Param name to index map
%(PARAMNAMES)s       
    }
    
    /// Destructor
    virtual ~ReactionNewtonSolver()
    {}

    /// Solve abstract nonlinear problem F(x) = 0 for given vector F and
    /// Jacobian dF/dx
    void solve(double dt, boost::shared_ptr<dolfin::GenericVector> xx)
    {

      // Get reference to the Vector
      dolfin::GenericVector& x = *xx.get();
      
      // Sanity check
      if (x.local_size() != _size * %(SYSTEMSIZE)s)
        error("expected a vector which local size is dividable with "
	      "the size of the system");
      
      // Grab parameters
      const uint maxiter = parameters["maximum_iterations"];
      const bool recompute_jacobian = parameters["recompute_jacobian"];
      const double relaxation = parameters["relaxation_parameter"];
      const std::string convergence_criterion = parameters["convergence_criterion"];
      const double rtol = parameters["relative_tolerance"];
      const double atol = parameters["absolute_tolerance"];
      const bool report = parameters["report"];
      const int num_threads = parameters["num_threads"];

      #ifdef HAS_OPENMP
      // Set number of OpenMP threads (from parameter systems)
      omp_set_num_threads(num_threads);
      #endif

%(SOLVE)s

    //info("Solution:");
    //for (int k=0; k<%(SYSTEMSIZE)s; k++)
    //  info("x[%%d]: %%f", k+619, x[_size*k+619]);
    }

    /// Set parameter
    void set_model_param(const std::string& name, double value)
    {
      std::cout << "Setting " << name << " to: " << value << std::endl;
      _params[_find_parameter(name)] = value;
    }

    /// Set vector params
    void set_model_vector_param(const std::string& name, 
				const std::vector<double>& values)
    {
      // Check size of values
      if (values.size() != _size)
	error("Expected the size of the vector parameters to be the same size of the system.");

      // Assign vector parameters
      _vector_params[_find_parameter(name)] = values;
      
    }

    // Method to solve one linear Newton iteration
    void solve_once(double dt, boost::shared_ptr<dolfin::GenericVector> xx0,
                    boost::shared_ptr<dolfin::GenericVector> xx)
    {

      // Get reference to the Vector
      dolfin::GenericVector& x = *xx.get();
      dolfin::GenericVector& x0 = *xx0.get();
      
      // Sanity check
      if (x.local_size() != _size * %(SYSTEMSIZE)s)
        error("expected a vector which local size is dvidable with "
	      "the size of the system");
      if (x0.local_size() != _size * %(SYSTEMSIZE)s)
        error("expected a vector which local size is dividable with "
	      "the size of the system");
      
      // Grab parameters
      const double relaxation = parameters["relaxation_parameter"];
      const int num_threads = parameters["num_threads"];

      #ifdef HAS_OPENMP
      // Set number of OpenMP threads (from parameter systems)
      omp_set_num_threads(num_threads);
      #endif

%(SOLVE_ONCE)s

    //info("Solution:");
    //for (int k=0; k<%(SYSTEMSIZE)s; k++)
    //  info("x[%%d]: %%f", k+619, x[_size*k+619]);
    }

  private:

%(PRIVATEFUNCS)s

    /// Check parameter name
    uint _find_parameter(const std::string& name) const
    {
      std::map<std::string, uint>::const_iterator it = _param_map.find(name);
      if (it == _param_map.end())
        throw std::runtime_error("\\"" + name + "\\" is not a parameter in this ReactionSolver");                         
      return it->second;
    }
    
    /// Size of the system
    uint _size;

    /// Parameter vector
    std::vector<double> _params;
    
    /// Parameter map
    std::map<std::string, uint> _param_map;

    /// Vector params
    std::map<uint, std::vector<double> > _vector_params;

  };
"""

reaction_form = {}.fromkeys(["PARAMNAMES", "PRIVATEFUNCS", "SOLVE_ONCE", "SOLVE",\
                             "SYSTEMSIZE", "NUMPARAMS", "COMPUTE_JACOBIAN", \
                             "SOLVE_U", "COMPUTE_F"], \
                             "NOT_ASSIGNED")

