__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2011-08-16 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

__all__ = ["DistanceToBoundaries"]

# System imports
import sympy, types

# SubCell imports
from subcell.common import *
from subcell.model.domains import MeshDomain
from subcell.model.boundaries import MeshBoundary
from subcell.model.species import CellParam, NodeParam
from subcell.model.subcellmodelobject import *

class DistanceToBoundaries(SubCellModelObject):
    def __init__(self, domain, *args):
        """
        Create an object which can be used to define a distance to a set
        of MeshBoundaries
        """
        check_arg(domain, MeshDomain)
        check_arg(args, tuple, itemtype=MeshBoundary)
        
        # Save attributes
        self.boundaries = args
        self.domain = domain
        
        # Collect boundary markers
        self.markers = []
        for boundary in self.boundaries:
            if domain not in boundary.domains:
                raise RuntimeError("expected all boundaries to be connected "\
                                   "with the domain : '%s'"%domain)
            for markers in boundary.markers:
                self.markers.extend(markers)
        
        name = "distance_to_%s"%("_".join(\
            str(boundary) for boundary in self.boundaries))
        
        # Create place holders for the cell and node distances
        params = odict()
        for dist_type, Param in zip(["node", "cell"], [NodeParam, CellParam]):
            dist_name = "%s_dist"%dist_type
            hidden_name = "__"+dist_name
            params[hidden_name] = create_parameter(\
                Param(domain, name=dist_name), dist_name, \
                property_param=False, allow_array=True)
            params[dist_name] = create_parameter(\
                params[hidden_name][0], dist_name, property_param=False, \
                allow_array=True, symname="%s_%s"%(name, dist_name))
        
        # Init base class
        SubCellModelObject.__init__(self, name, params)
        
    def __repr__(self):
        return "DistanceToBoundaries(%s)"%(", ".join(\
            repr(boundary) for boundary in self.boundaries))
