__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-10-18 -- 2011-08-18"
__license__  = "GNU LGPL Version 3.0 or later"

__all__ = ["load_model", "get_present_subcellmodeleditor", \
           "get_load_arguments"]

# System imports
import inspect
import sympy
import types

# Import subcell modules
from subcell.common import camel_capitalize, RangeCheck, TypeCheck
from subcellmodel import SubCellModel
from subcellmodeleditor import SubCellModelEditor
from subcellmodelobject import SubCellModelObject

# Temp variable for the present subcellmodel
__editor = SubCellModelEditor()
__load_arguments = None

def load_model(model_name, **kwargs):
    """
    Load a SubCellModel from file and return the instance

    The method looks for a file with .scm extension.

    """
    import modellanguage

    global __subcellmodel, __load_arguments

    # If a TypeCheck is provided turn it into its value
    for key, value in kwargs.items():
        if isinstance(value, TypeCheck):
            kwargs[key] = value.get()

    __load_arguments = kwargs

    # Create the SubCellModel editor which shall be populated by the form file
    __subcellmodel = SubCellModel(model_name)
    __editor.open(__subcellmodel)
    reload(modellanguage)
    
    # Execute the file in the modellanguage module
    execfile(model_name + ".scm", modellanguage.__dict__)

    # Close the model
    __editor.close()

    return __subcellmodel

def get_present_subcellmodeleditor():
    """
    Return the present subcellmodel
    """
    global __editor
    return __editor

def get_load_arguments():
    """
    Return the present load_arguments
    """

    global __load_arguments
    return __load_arguments
