__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-12-01 -- 2011-09-20"
__license__  = "GNU LGPL Version 3.0 or later"

__all__ = ["StaticElectricPotential"]

# System imports
import sympy
import numpy

# subcell imports
from subcell.common import *

# Local imports
from subcellmodelobject import *
from domains import MeshDomain
from variables import Variable
from species import *

class StaticElectricPotential(object):
    "A class for a user specified StaticElectricPotential"
    def __init__(self, expr, domain):
        check_arg(expr, (sympy.Basic, TypeCheck), 1)
        check_arg(domain, MeshDomain, 2)
        
        # If setting the potential using a SymPy expression
        if isinstance(expr, sympy.Basic):
            
            # Check that the expr is static and Coordinate dependent
            not_allowed = (Variable, DomainSpecificSpecies)
            
            for atom in expr.atoms():
                if isinstance(atom, not_allowed):
                    raise TypeError, "A StaticElectricPotential cannot "\
                          "include a variable or species, only constants and "\
                          "coordinates."
                
                if isinstance(atom, ParSymbol) and \
                       (not atom.param.is_node_param or \
                        atom.param.domain != domain):
                    raise TypeError("Any NodeParam which defines the "
                            "'StaticElectricPotential' need to be defined "\
                            "at the same domain as the Potential")
                
                
        # else a NodeParam is used
        elif expr.is_node_param:
            if expr.domain != domain:
                raise TypeError("Any NodeParam which defines the "
                                "'StaticElectricPotential' need to be defined"\
                                " at the same domain as the Potential")
        else:
            raise TypeError, "expected a NodeParameters as the 'expr'"
        
        # Store attributes
        self.domain = domain
        self.expr = ParSlave(expr)
