"""Run all demos."""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__date__ = "2010-03-19 -- 2011-08-11"
__copyright__ = "Copyright (C) 2010 " + __author__
__license__  = "GNU LGPL version 3.0"

from time import time

from subcell.common import get_status_output
import re, sys, os, platform

# Location of all demos
demodir = os.path.join(os.curdir, "..", "..", "demo")

# Demos to skip
skip_demos = []

# Demos to run
pydemos = []
for dpath, dnames, fnames in os.walk(demodir):
    if any(skip_demo in dpath for skip_demo in skip_demos):
        continue
    if os.path.isfile(os.path.join(dpath, 'demo.py')):
        pydemos.append(dpath)

# Set non-interactive
os.putenv('SUBCELL_NOPLOT', '1')

print "Running all demos"
print ""
print "Found %d Python demos" % len(pydemos)
print ""
import pprint

failed = []
timing = []

sys

total_demos = 0
options = [""]

# Run Python demos
for demo in pydemos:
    if os.path.isfile(os.path.join(demo, 'demo.py')):
        for option in options:
            total_demos += 1
            print "----------------------------------------------------------------"
            print "Running Python demo %s %s" % (demo, option)
            print ""
            t1 = time()
            output = get_status_output("cd %s && python demo.py %s" % (demo, option))
            t2 = time()
            timing += [(t2 - t1, (demo + " " + option).strip())]
            if output[0] == 0:
                print "OK"
            elif output[0] == 10: # Failing but exiting gracefully
                print "ok (graceful exit on fail)"
            else:
                print "*** Failed"
                print output[1]
                failed += [(demo, option, output[1])]
    else:
        print "*** Warning: missing demo"

# Print summary of time to run demos
timing.sort()
print ""
print "Time to run demos:"
print "\n".join("%.2fs: %s" % t for t in timing)

# Print output for failed tests
print ""
if len(failed) > 0:
    print "%d demo(s) out of %d failed, see demo.log for details." % \
          (len(failed), total_demos)
    file = open("demo.log", "w")
    for (test, interface, output) in failed:
        file.write("----------------------------------------------------------------------\n")
        file.write("%s (%s)\n" % (test, interface))
        file.write("\n")
        file.write(output)
        file.write("\n")
        file.write("\n")
else:
    print "All demos checked: OK"

# Return error code if tests failed
sys.exit(len(failed) != 0)
